#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Plotting vector data.

This module can be used in two different ways:

1. As a library. Just import the module and call the functions.
   This is the way, how this module is used in openpivgui, for
   example.

2. As a terminal-application. Execute
   python3 -m openpivgui.vec_plot --help
   for more information.
   This is the way, how this module ist used in JPIV, for example.
   For now, not all functions are callable in this way.
"""

import numpy as np
from matplotlib import pyplot as plt
import matplotlib
from matplotlib.colors import LinearSegmentedColormap
from matplotlib.figure import Figure
import argparse
__licence__ = '''
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

__email__ = 'vennemann@fh-muenster.de'


# creating a custom rainbow colormap

# creating a custom rainbow colormap
short_rainbow = {'red': ((0.0, 0.0, 0.0),
                         (0.2, 0.2, 0.2),
                         (0.5, 0.0, 0.0),
                         (0.8, 1.0, 1.0),
                         (1.0, 1.0, 1.0)),
                 'green': ((0.0, 0.0, 0.0),
                           (0.2, 1.0, 1.0),
                           (0.5, 1.0, 1.0),
                           (0.8, 1.0, 1.0),
                           (1.0, 0.0, 0.0)),
                 'blue': ((0.0, 1.0, 1.0),
                          (0.2, 1.0, 1.0),
                          (0.5, 0.0, 0.0),
                          (0.8, 0.0, 0.0),
                          (1.0, 0.0, 0.0))}

long_rainbow = {'red': ((0.0, 0.0, 0.0),
                        (0.1, 0.5, 0.5),
                        (0.2, 0.0, 0.0),
                        (0.3, 0.2, 0.2),
                        (0.5, 0.0, 0.0),
                        (0.7, 1.0, 1.0),
                        (0.8, 1.0, 1.0),
                        (1.0, 1.0, 1.0)),
                'green': ((0.0, 0.0, 0.0),
                          (0.1, 0.0, 0.0),
                          (0.2, 0.0, 0.0),
                          (0.3, 1.0, 1.0),
                          (0.5, 1.0, 1.0),
                          (0.7, 1.0, 1.0),
                          (0.8, 0.0, 0.0),
                          (1.0, 0.3, 0.3)),
                'blue': ((0.0, 0.0, 0.0),
                         (0.1, 0.5, 0.5),
                         (0.2, 1.0, 1.0),
                         (0.3, 1.0, 1.0),
                         (0.5, 0.0, 0.0),
                         (0.7, 0.0, 0.0),
                         (0.8, 0.0, 0.0),
                         (1.0, 1.0, 1.0))}

short_rainbow = LinearSegmentedColormap('my_colormap', short_rainbow, 256)
long_rainbow = LinearSegmentedColormap('my_colormap', long_rainbow, 256)


def histogram(data, parameter, figure, quantity, bins, log_y):
    """
        Plot an histogram.

        Plots an histogram of the specified quantity.

        Parameters
        ----------
        data : pandas.DataFrame
            Data to plot.
        figure : matplotlib.figure.Figure
            An (empty) Figure object.
        quantity : str
            Either v (abs v), v_x (x-component) or v_y (y-component).
        bins : int
             Number of bins (bars) in the histogram.
        log_scale : boolean
            Use logaritmic vertical axis.
    """

    if quantity == 'v':
        xlabel = 'absolute displacement'
        h_data = np.array([(l[2]**2 + l[3]**2)**0.5 for l in data])
    elif quantity == 'v_x':
        xlabel = 'x displacement'
        h_data = np.array([l[2] for l in data])
    elif quantity == 'v_y':
        xlabel = 'y displacement'
        h_data = np.array([l[3] for l in data])
    ax = figure.add_subplot(111)
    if log_y:
        ax.set_yscale("log")
    ax.hist(h_data, bins, label=quantity)
    ax.set_xlabel(xlabel)
    ax.set_ylabel('number of vectors')
    ax.set_title(parameter['plot_title'])


def profiles(data, parameter, fname, figure, orientation):
    """
        Plot velocity profiles.

        Line plots of the velocity component specified.

        Parameters
        ----------
        data : pandas.DataFrame
            Data to plot.
        fname : str
            A filename containing vector data.
            (will be deprecated in later updates)
        figure : matplotlib.figure.Figure
            An (empty) Figure object.
        orientation : str
            horizontal: Plot v_y over x.
            vertical: Plot v_x over y.
    """
    #data = data.to_numpy().astype(np.float)
    data = np.loadtxt(fname)

    dim_x, dim_y = get_dim(data)

    p_data = []

    ax = figure.add_subplot(111)

    if orientation == 'horizontal':
        xlabel = 'x position'
        ylabel = 'y displacement'

        for i in range(0, dim_y, parameter['profiles_jump']):
            p_data.append(data[dim_x * i:dim_x * (i + 1), 3])
        # print(p_data[-1])
        for p in p_data:
            # print(len(p))
            ax.plot(range(dim_x), p, '.-')

    elif orientation == 'vertical':
        xlabel = 'y position'
        ylabel = 'x displacement'

        for i in range(0, dim_x, parameter['profiles_jump']):
            p_data.append(data[i::dim_x, 2])

        for p in p_data:
            ax.plot(range(dim_y), p, '.-')

    ax.set_xlabel(xlabel)
    ax.set_ylabel(ylabel)
    ax.set_title(parameter['plot_title'])


def scatter(data, figure):
    """
        Scatter plot.

        Plots v_y over v_x.

        Parameters
        ----------
        data : pandas.DataFrame
            Data to plot.
        figure : matplotlib.figure.Figure
            An (empty) Figure object.
    """
    data = data.to_numpy()

    v_x = data[:, 2]
    v_y = data[:, 3]

    ax = figure.add_subplot(111)

    ax.scatter(v_x, v_y, label='scatter')

    ax.set_xlabel('x displacement')
    ax.set_ylabel('y displacement')


def vector(data, parameter, figure, invert_yaxis=True, valid_color='blue',
           invalid_color='red', **kw):
    """
        Display a vector plot.

        Parameters
        ----------
        data : pandas.DataFrame
            Data to plot.
        figure : matplotlib.figure.Figure
            An (empty) Figure object.
    """
    data = data.to_numpy().astype(float)

    try:
        invalid = data[:, 4].astype('bool')
    except BaseException:
        invalid = np.asarray([True for i in range(len(data))])

    # tilde means invert:
    valid = ~invalid

    ax = figure.add_subplot(111)

    ax.quiver(data[invalid, 0],
              data[invalid, 1],
              data[invalid, 2],
              data[invalid, 3],
              color=invalid_color,
              label='invalid', **kw)

    ax.quiver(data[valid, 0],
              data[valid, 1],
              data[valid, 2],
              data[valid, 3],
              color=valid_color,
              label='valid', **kw)

    if invert_yaxis:
        for ax in figure.get_axes():
            ax.invert_yaxis()

    ax.set_xlabel('x position')
    ax.set_ylabel('y position')
    ax.set_title(parameter['plot_title'])


def contour(data, parameter, figure):
    """
        Display a contour plot

        Parameters
        ----------
        data : pandas.DataFrame
            Data to plot.
        parameter : openpivgui.OpenPivParams.py
            Parameter-object.
        figure : matplotlib.figure.Figure
           An (empty) Figure object.
    """
    # figure for subplot
    ax = figure.add_subplot(111)
    # iteration to set value types to float
    for i in list(data.columns.values):
        data[i] = data[i].astype(float)
    # choosing velocity for the colormap and add it to an new column in data
    if parameter['velocity_color'] == 'vx':
        data['abs'] = data.vx
    elif parameter['velocity_color'] == 'vy':
        data['abs'] = data.vy
    else:
        data['abs'] = (data.vx**2 + data.vy**2)**0.5
    # pivot table for contour function
    data_pivot = data.pivot(index='y',
                            columns='x',
                            values='abs')
    # try to get limits, if not possible set to None
    try:
        vmin = float(parameter['vmin'])
    except BaseException:
        vmin = None
    try:
        vmax = float(parameter['vmax'])
    except BaseException:
        vmax = None
    # settings for color scheme of the contour plot
    if vmax is not None and vmin is not None:
        levels = np.linspace(vmin, vmax, int(parameter['color_levels']))
    elif vmax is not None:
        levels = np.linspace(0, vmax, int(parameter['color_levels']))
    elif vmin is not None:
        vmax = data_pivot.max().max()
        levels = np.linspace(vmin, vmax, int(parameter['color_levels']))
    else:
        levels = int(parameter['color_levels'])
    # Choosing the correct colormap
    if parameter['color_map'] == 'short rainbow':
        colormap = short_rainbow
    elif parameter['color_map'] == 'long rainbow':
        colormap = long_rainbow
    else:
        colormap = parameter['color_map']
    # set contour plot to the variable fig to add a colorbar
    if parameter['extend_cbar']:
        extend = 'both'
    else:
        extend = None
    fig = ax.contourf(data_pivot.columns,
                      data_pivot.index,
                      data_pivot.values,
                      levels=levels,
                      cmap=colormap,
                      vmin=vmin,
                      vmax=vmax,
                      extend=extend)

    # set the colorbar to the variable cb to add a description
    cb = plt.colorbar(fig, ax=ax)

    # set origin to top left or bottom left
    if parameter['invert_yaxis']:
        ax.set_ylim(ax.get_ylim()[::-1])

    # description to the contour lines
    cb.ax.set_ylabel('Velocity [m/s]')

    # labels for the axes
    ax.set_xlabel('x-position')
    ax.set_ylabel('y-position')

    # plot title from the GUI
    ax.set_title(parameter['plot_title'])


def contour_and_vector(data, parameter, figure, **kw):
    """
        Display a contour plot

        Parameters
        ----------
        data : pandas.DataFrame
            Data to plot.
        parameter : openpivgui.OpenPivParams.py
            Parameter-object.
        figure : matplotlib.figure.Figure
           An (empty) Figure object.
    """
    # figure for subplot
    ax = figure.add_subplot(111)
    # iteration to set value types to float
    for i in list(data.columns.values):
        data[i] = data[i].astype(float)
    # choosing velocity for the colormap and add it to an new colummn in data
    if parameter['velocity_color'] == 'vx':
        data['abs'] = data.vx
    elif parameter['velocity_color'] == 'vy':
        data['abs'] = data.vy
    else:
        data['abs'] = (data.vx**2 + data.vy**2)**0.5
    # pivot table for contour function
    data_pivot = data.pivot(index='y',
                            columns='x',
                            values='abs')
    # try to get limits, if not possible set to None
    try:
        vmin = float(parameter['vmin'])
    except BaseException:
        vmin = None
    try:
        vmax = float(parameter['vmax'])
    except BaseException:
        vmax = None
    # settings for color scheme of the contour plot
    if vmax is not None and vmin is not None:
        levels = np.linspace(vmin, vmax, int(parameter['color_levels']))
    elif vmax is not None:
        levels = np.linspace(0, vmax, int(parameter['color_levels']))
    elif vmin is not None:
        vmax = data_pivot.max().max()
        levels = np.linspace(vmin, vmax, int(parameter['color_levels']))
    else:
        levels = int(parameter['color_levels'])
    # Choosing the correct colormap
    if parameter['color_map'] == 'short rainbow':
        colormap = short_rainbow
    elif parameter['color_map'] == 'long rainbow':
        colormap = long_rainbow
    else:
        colormap = parameter['color_map']

    # set contour plot to the variable fig to add a colorbar
    if parameter['extend_cbar']:
        extend = 'both'
    else:
        extend = None
    fig = ax.contourf(data_pivot.columns,
                      data_pivot.index,
                      data_pivot.values,
                      levels=levels,
                      cmap=colormap,
                      vmin=vmin,
                      vmax=vmax,
                      extend=extend)

    # quiver plot
    data = data.to_numpy().astype(float)

    try:
        invalid = data[:, 4].astype('bool')
    except BaseException:
        invalid = np.asarray([True for i in range(len(data))])

    # tilde means invert:
    valid = ~invalid

    ax.quiver(data[invalid, 0],
              data[invalid, 1],
              data[invalid, 2],
              data[invalid, 3],
              color=parameter['invalid_color'],
              label='invalid', **kw)

    ax.quiver(data[valid, 0],
              data[valid, 1],
              data[valid, 2],
              data[valid, 3],
              color=parameter['valid_color'],
              label='valid', **kw)

    # set the colorbar to the variable cb to add a description
    cb = plt.colorbar(fig, ax=ax)

    # set origin to top left or bottom left
    if parameter['invert_yaxis']:
        ax.set_ylim(ax.get_ylim()[::-1])

    # description to the contour lines
    cb.ax.set_ylabel('Velocity [m/s]')

    # labels for the axes
    ax.set_xlabel('x-position')
    ax.set_ylabel('y-position')

    # plot title from the GUI
    ax.set_title(parameter['plot_title'])


def get_dim(array):
    """
        Computes dimension of vector data.

        Assumes data to be organised as follows (example):
        x  y  v_x v_y ..
        16 16 4.5 3.2 ..
        32 16 4.3 3.1 ..
        16 32 4.2 3.5 ..
        32 32 4.5 3.2 ..
        .. .. ..  ..

        Parameters
        ----------
        array : np.array
            Flat numpy array.

        Returns
        -------
        tuple
            Dimension of the vector field (x, y).
    """
    return(len(set(array[:, 0])),
           len(set(array[:, 1])))


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description='Plot vector data.')
    parser.add_argument('--plot_type',
                        type=str,
                        required=False,
                        choices=['histogram',
                                 'profiles',
                                 'vector',
                                 'scatter',
                                 'contour'
                                 'contour_and_vector',
                                 'streamlines'],
                        default='vector',
                        help='type of plot')
    parser.add_argument('--fname',
                        required=True,
                        type=str,
                        help='name of vector data file')
    parser.add_argument('--quantity',
                        type=str,
                        required=False,
                        choices=['v', 'v_x', 'v_y'],
                        default='v',
                        help='quantity to plot')
    parser.add_argument('--bins',
                        type=int,
                        required=False,
                        default=20,
                        help='number of histogram bins')
    parser.add_argument('--log_y',
                        type=bool,
                        required=False,
                        default=False,
                        help='logarithmic y-axis')
    parser.add_argument('--orientation',
                        type=str,
                        required=False,
                        choices=['horizontal', 'vertical'],
                        default='vertical',
                        help='plot profiles, either horizontal ' +
                             '(v_y over x) or vertical (v_x over y)')
    parser.add_argument('--invert_yaxis',
                        type=str,
                        required=False,
                        default=True,
                        help='Invert y-axis of vector plot')
    args = parser.parse_args()
    data = np.loadtxt(args.fname)
    fig = Figure()
    if args.plot_type == 'histogram':
        histogram(data,
                  fig,
                  quantity=args.quantity,
                  bins=args.bins,
                  log_y=args.log_y)
    elif args.plot_type == 'profiles':
        profiles(data,
                 fig,
                 orientation=args.orientation)
    elif args.plot_type == 'vector':
        vector(data,
               fig,
               invert_yaxis=args.invert_yaxis)
    elif args.plot_type == 'scatter':
        scatter(data,
                fig)
    elif args.plot_type == 'contour':
        print('Not yet implemented')
    elif args.plot_type == 'contour_and_vector':
        print('Not yet implemented')
    elif args.plot_type == 'streamlines':
        print('Not yet implemented')
    plt.show()



class AddIn:
    variables = {}
    addin_tip = ""
    add_in_name = ""

    def get_variables(self):
        return self.variables

    def get_description(self):
        return self.addin_tip

    def __init__(self):
        print("Initializing " + self.add_in_name)



from openpivgui.AddIns.AddIn import AddIn
import numpy as np
from openpivgui.open_piv_gui_tools import create_save_vec_fname, save
import openpiv.filters as piv_flt


class repl_outliers_addin_postprocessing(AddIn):
    """
        Blueprint for developing own methods and inserting own variables
        into the already existing PIV GUI via the AddIn system
    """

    # description for the Add_In_Handler textarea
    addin_tip = "This is the description of the advanced filter addin which " \
                "is still missing now"

    # has to be the add_in_name and its abbreviation
    add_in_name = "repl_outliers_addin (roa)"

    # variables
    #########################################################
    # Place additional variables in the following sections. #
    # Widgets are created automatically. Don't care about   #
    # saving and restoring - new variables are included     #
    # automatically.                                        #
    #                                                       #
    # e.g.                                                  #
    #   **abbreviation**_**variable_name** =                #
    #       [**id over super group**, **variable_type**,    #
    #        **standard_value**,**hint**, **label**         #
    #        **tool tip**                                   #
    #########################################################
    variables = {
        'roa_repl':
        [7010, 'bool', True, None,
         'replace outliers',
         'Replace outliers.'],

            'roa_repl_method':
                [7020, 'str', 'localmean',
                 ('localmean', 'disk', 'distance'),
                 'replacement method',
                 'Each NaN element is replaced by a weighed average' +
                 'of neighbours. Localmean uses a square kernel, ' +
                 'disk a uniform circular kernel, and distance a ' +
                 'kernel with a weight that is proportional to the ' +
                 'distance.'],

            'roa_repl_iter':
                [7030, 'int', 10, None,
                 'number of iterations',
                 'If there are adjacent NaN elements, iterative ' +
                 'replacement is needed.'],

            'roa_repl_kernel':
                [7040, 'int', 2, None,
                 'kernel size',
                 'Diameter of the weighting kernel.'],

            'roa_horizontal_spacer15':
                [7045, 'h-spacer', None, None, None, None]
    }

    def repl_outliers(self, gui, delimiter):
        """Replace outliers."""
        result_fnames = []
        for i, f in enumerate(gui.p['fnames']):
            data = np.loadtxt(f)
            u, v = piv_flt.replace_outliers(
                np.array([data[:, 2]]), np.array([data[:, 3]]),
                method=gui.p['roa_repl_method'],
                max_iter=gui.p['roa_repl_iter'],
                kernel_size=gui.p['roa_repl_kernel'])
            save_fname = create_save_vec_fname(
                path=f,
                postfix='_repl')
            save(data[:, 0],
                 data[:, 1],
                 u, v,
                 data[:, 4],
                 data[:, 5],
                 save_fname,
                 delimiter=delimiter)
            result_fnames.append(save_fname)
        return result_fnames

    def __init__(self, gui):
        super().__init__()
        # has to be the method which is implemented above
        gui.postprocessing_methods.update(
            {"repl_outliers_addin_postprocessing":
             ['postprocessing', 'roa_repl', self.repl_outliers]})



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

''' Methods for reuse within the OpenPivGui project.'''

import numpy as np
import math
import os
__licence__ = '''
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

__email__ = 'vennemann@fh-muenster.de'


def str2list(s):
    '''Parses a string representation of a list.

    Parameters
    ----------
    s : str
        String containing comma separated values.

    Example
    -------
    str2list('img01.png', 'img02.png')

    Returns
    -------
    list
    '''
    return([t.strip("' ") for t in s.strip('(),').split(',')])


def str2dict(s):
    '''Parses a string representation of a dictionary.

    Parameters
    ----------
    s : str
        Comma separated list of colon separated key value pairs.

    Example
    -------
    str2dict('key1: value1', 'key2: value2')
    '''
    d = {}
    l = str2list(s)
    for elem in l:
        key, value = elem.split(':')
        key.strip(' ')
        value.strip(' ')
        d.update({key: value})
    return(d)


def create_save_vec_fname(path=os.getcwd(),
                          basename=None,
                          postfix='',
                          count=-1,
                          max_count=9):
    '''Assembles a valid absolute path for saving vector data.

    Parameters
    ----------
    path : str
        Directory path. Default: Working directory.
    basename : str
        Prefix. Default: None.
    postfix : str
        Postfix. Default: None.
    count : int
        Counter for numbering filenames.
        Default: -1 (no number)
    max_count : int
        Highest number to expect. Used for generating
        leading zeros. Default: 9 (no leading zeros).
    '''
    if count == -1:
        num = ''
    else:
        num = str(count).zfill(math.ceil(math.log10(max_count)))
    if basename is None:
        basename = os.path.basename(path)
    elif basename == '':
        basename = 'out'
    return(os.path.dirname(path) +
           os.sep +
           basename.split('.')[0] +
           postfix +
           num +
           '.vec')


def get_dim(array):
    '''Computes dimension of vector data.

    Assumes data to be organised as follows (example):
    x  y  v_x v_y
    16 16 4.5 3.2
    32 16 4.3 3.1
    16 32 4.2 3.5
    32 32 4.5 3.2

    Parameters
    ----------
    array : np.array
        Flat numpy array.

    Returns
    -------
    tuple
        Dimension of the vector field (x, y).
    '''
    return(len(set(array[:, 0])),
           len(set(array[:, 1])))


def save(x, y, u, v, mask, sig2noise, filename, fmt='%8.4f', delimiter='\t'):
    out = np.vstack([m.ravel() for m in [x, y, u, v, mask, sig2noise]])
    np.savetxt(filename, out.T, fmt=fmt, delimiter=delimiter)


def _round(number, decimals=0):
    multiplier = 10 ** decimals
    return(math.floor(number * multiplier + 0.5) / multiplier)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Post Processing for OpenPIVGui."""

from openpivgui.open_piv_gui_tools import create_save_vec_fname, save
import openpiv.smoothn as piv_smt
import openpiv.filters as piv_flt
import openpiv.validation as piv_vld
import openpiv.tools as piv_tls
import numpy as np
__licence__ = '''
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

__email__ = 'vennemann@fh-muenster.de'


class PostProcessing:
    """Post Processing routines for vector data.

    Parameters
    ----------
    params : openpivgui.OpenPivParams
        Parameter object.
    """

    delimiter = ''

    def __init__(self, params):
        """Initialization method."""
        self.p = params

        self.delimiter = self.p['delimiter']
        if self.delimiter == 'tab':
            self.delimiter = '\t'
        if self.delimiter == 'space':
            self.delimiter = ' '

    def sig2noise(self):
        """Filter vectors based on the signal to noise threshold.

        See:
            openpiv.validation.sig2noise_val()
        """
        result_fnames = []
        for i, f in enumerate(self.p['fnames']):
            data = np.loadtxt(f)
            u, v, mask = piv_vld.sig2noise_val(
                data[:, 2], data[:, 3], data[:, 5],
                threshold=self.p['sig2noise_threshold'])

            save_fname = create_save_vec_fname(
                path=f,
                postfix='_sig2noise')

            save(data[:, 0],
                 data[:, 1],
                 u, v,
                 data[:, 4] + mask,
                 sig2noise=data[:, 5],
                 filename=save_fname,
                 delimiter=self.delimiter)
            result_fnames.append(save_fname)
        return result_fnames

    def global_std(self):
        """
            Filters vectors by a multiple of the standard deviation.

            See Also
            --------
            openpiv.validation.global_std()
        """
        result_fnames = []
        for i, f in enumerate(self.p['fnames']):
            data = np.loadtxt(f)
            u, v, mask = piv_vld.global_std(
                data[:, 2], data[:, 3],
                std_threshold=self.p['global_std_threshold'])
            save_fname = create_save_vec_fname(
                path=f,
                postfix='_std_thrhld')
            save(data[:, 0],
                 data[:, 1],
                 u, v,
                 data[:, 4] + mask,
                 data[:, 5],
                 save_fname,
                 delimiter=self.delimiter)
            result_fnames.append(save_fname)
        return result_fnames

    def global_val(self):
        """
            Filter vectors based on a global min-max threshold.

            See:
                openpiv.validation.global_val()
        """
        result_fnames = []
        for i, f in enumerate(self.p['fnames']):
            data = np.loadtxt(f)
            u, v, mask = piv_vld.global_val(
                data[:, 2], data[:, 3],
                u_thresholds=(self.p['MinU'], self.p['MaxU']),
                v_thresholds=(self.p['MinV'], self.p['MaxV']))
            save_fname = create_save_vec_fname(
                path=f,
                postfix='_glob_thrhld')
            save(data[:, 0],
                 data[:, 1],
                 u, v,
                 data[:, 4] + mask,
                 data[:, 5],
                 save_fname,
                 delimiter=self.delimiter)
            result_fnames.append(save_fname)
        return result_fnames

    def local_median(self):
        """
            Filter vectors based on a local median threshold.

            See Also
            --------
            openpiv.validation.local_median_val()
        """
        result_fnames = []
        for i, f in enumerate(self.p['fnames']):
            data = np.loadtxt(f)
            u, v, mask = piv_vld.local_median_val(
                data[:, 2], data[:, 3],
                u_threshold=self.p['local_median_threshold'],
                v_threshold=self.p['local_median_threshold'],
                size=self.p['local_median_size'])
            save_fname = create_save_vec_fname(
                path=f,
                postfix='_med_thrhld')
            save(data[:, 0],
                 data[:, 1],
                 u, v,
                 data[:, 4] + mask,
                 data[:, 5],
                 save_fname,
                 delimiter=self.delimiter)
            result_fnames.append(save_fname)
        return result_fnames

    def repl_outliers(self):
        """Replace outliers."""
        result_fnames = []
        for i, f in enumerate(self.p['fnames']):
            data = np.loadtxt(f)
            u, v = piv_flt.replace_outliers(
                np.array([data[:, 2]]), np.array([data[:, 3]]),
                method=self.p['repl_method'],
                max_iter=self.p['repl_iter'],
                kernel_size=self.p['repl_kernel'])
            save_fname = create_save_vec_fname(
                path=f,
                postfix='_repl')
            save(data[:, 0],
                 data[:, 1],
                 u, v,
                 data[:, 4],
                 data[:, 5],
                 save_fname,
                 delimiter=self.delimiter)
            result_fnames.append(save_fname)
        return result_fnames

    def smoothn_r(self):
        """Smoothn postprocessing results."""
        result_fnames = []
        for i, f in enumerate(self.p['fnames']):
            data = np.loadtxt(f)
            u, _, _, _ = piv_smt.smoothn(
                data[:, 2], s=self.p['smoothn_val'], isrobust=self.p['robust'])
            v, _, _, _ = piv_smt.smoothn(
                data[:, 3], s=self.p['smoothn_val'], isrobust=self.p['robust'])
            save_fname = create_save_vec_fname(
                path=f,
                postfix='_smthn')
            save(data[:, 0],
                 data[:, 1],
                 u, v,
                 data[:, 4],
                 data[:, 5],
                 save_fname,
                 delimiter=self.delimiter)
            result_fnames.append(save_fname)
        return result_fnames

    def average(self):
        """Average all results."""
        """data = np.loadtxt(self.p['fnames'][0])
        u = data[:, 2]
        v = data[:, 3]
        for i, f in enumerate(self.p['fnames']):
            data = np.loadtxt(f)
            u += data[:,2]; u /= 2
            v += data[:,3]; v /= 2
        save_fname = create_save_vec_fname(
            path=self.p['fnames'][0],
            postfix='_average')
        piv_tls.save(data[:, 0],
                     data[:, 1],
                     u, v,
                     data[:, 4],
                     save_fname,
                     delimiter = self.delimiter)
        print(f)
        return(save_fname)"""
        return 'Averaging of vectors fileds is not implemented.'



from openpivgui.AddIns.AddIn import AddIn
import tkinter.messagebox as messagebox
import tkinter as tk
import tkinter.ttk as ttk


def user_function(gui):
    """
        Executes user function.
    """
    gui.get_settings()
    print(gui.p['ufa_addin_user_func_def'])
    exec(gui.p['ufa_addin_user_func_def'])


def create_user_function_buttons(gui, menu):
    """ adds the buttons to the gui that are needed for the user func"""
    menu.add_command(label='Show User Function',
                     command=lambda: gui.selection(10))
    menu.add_command(label='Execute User Function',
                     command=lambda: user_function(gui))


class user_function_addin_other(AddIn):

    example_user_function = '''
filelistbox = gui.get_filelistbox()
properties  = gui.p
import pandas as pd

def textbox(title='Title', text='Hello!'):
    from tkinter.scrolledtext import ScrolledText
    from tkinter.constants import END
    frame = tk.Tk()
    frame.title(title)
    textarea = ScrolledText(frame, height=10, width=80)
    textarea.insert(END, text)
    textarea.pack(fill='x', side='left', expand=True)
    textarea.focus()
    frame.mainloop()

try:
    index = filelistbox.curselection()[0]
except IndexError:
    messagebox.showerror(
        title="No vector file selected.",
        message="Please select a vector file " +
                "in the file list and run again."
    )
else:
    f = properties['fnames'][index]
    names=('x','y','v_x','v_y','var')
    df = pd.read_csv(f, sep='\t', header=None, names=names)
    print(df.describe())
    textbox(title='Statistics of {}'.format(f),
            text=df.describe()
    )
'''

    variables = {'ufa_addin_user_func':
                 [10000, None, None, None, 'User-Function', None],
                 'ufa_addin_user_func_def':
                     [10010, 'text', example_user_function, None, None, None]}

    add_in_name = "user_function_addin (ufa)"

    def __init__(self, gui):
        super().__init__()
        gui.buttons.update({"user_function_addin_other":
                            create_user_function_buttons})



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""Post Processing for OpenPIVGui."""

from skimage import exposure, filters, util
import openpiv.preprocess as piv_pre
import openpiv.tools as piv_tls
import numpy as np

__licence__ = '''
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.
This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
'''

__email__ = 'vennemann@fh-muenster.de'

'''Pre Processing chain for image arrays.

Parameters
----------
params : openpivgui.OpenPivParams
    Parameter object.
'''


def gen_background(self, image1=None, image2=None):
    self.p = self
    images = self.p['fnames'][self.p['starting_frame']: self.p['ending_frame']]
    # This needs more testing. It creates artifacts in the correlation
    # for images not selected in the background.
    if self.p['background_type'] == 'global min':
        background = piv_tls.imread(self.p['fnames'][self.p['starting_frame']])
        maximum = background.max()
        background = background / maximum
        background *= 255
        for im in images:
            # the original image is already included, so skip it in the
            # for loop
            if im == self.p['fnames'][self.p['starting_frame']]:
                pass
            else:
                image = piv_tls.imread(im)
                maximum = image.max()
                image = image / maximum
                image *= 255
                background = np.min(np.array([background, image]), axis=0)
        return background

    elif self.p['background_type'] == 'global mean':
        images = self.p['fnames'][self.p['starting_frame']:
                                  self.p['ending_frame']]
        background = piv_tls.imread(self.p['fnames'][self.p['starting_frame']])
        maximum = background.max()
        background = background / maximum
        background *= 255
        for im in images:
            # the original image is already included, so skip it in the
            # for loop
            if im == self.p['fnames'][self.p['starting_frame']]:
                pass
            else:
                image = piv_tls.imread(im)
                maximum = image.max()
                image = image / maximum
                image *= 255
                background += image
        background /= (self.p['ending_frame'] - self.p['starting_frame'])
        return background

    elif self.p['background_type'] == 'minA - minB':
        # normalize image1 and image2 intensities to [0,255]
        maximum1 = image1.max()
        maximum2 = image2.max()
        image1 = image1 / maximum1
        image2 = image2 / maximum2
        image1 *= 255
        image2 *= 255
        background = np.min(np.array([image2, image1]), axis=0)
        return background

    else:
        print('Background algorithm not implemented.')


def process_images(self, img, preprocessing_methods, background=None):
    """Starting the pre-processing chain"""
    # normalize image to [0, 1] float
    maximum = img.max()
    img = img / maximum
    resize = self.p['img_int_resize']
    if self.p['invert']:
        img = util.invert(img)

    if self.p['background_subtract']:
        try:
            img *= 255
            img -= background
            img[img < 0] = 0  # values less than zero are set to zero
            img = img / 255
        except BaseException:
            print('Could not subtract background. Ignoring background '
                  'subtraction.')
    # ROI crop done after background subtraction to avoid image shape issues
    if self.p['crop_ROI']:
        crop_x = (int(list(self.p['crop_roi-xminmax'].split(','))[0]),
                  int(list(self.p['crop_roi-xminmax'].split(','))[1]))
        crop_y = (int(list(self.p['crop_roi-yminmax'].split(','))[0]),
                  int(list(self.p['crop_roi-yminmax'].split(','))[1]))
        img = img[crop_y[0]:crop_y[1], crop_x[0]:crop_x[1]]

    # if self.p['dynamic_mask']: # needs more testing
    #    img = piv_pre.dynamic_masking(img,
    #                                  method=self.p['dynamic_mask_type'],
    #                                  filter_size=self.p['dynamic_mask_size'],
    #                                  threshold=self.p[
    #                                  'dynamic_mask_threshold'])

    # this for loop is used to load the methods stored in the Add_ins
    # the add_ins have to end with _preprocessing to be loaded here
    for method in preprocessing_methods:
        img = preprocessing_methods[method](img, self)

    return img * resize



from openpivgui.AddIns.AddIn import AddIn
from scipy.ndimage.filters import gaussian_filter
from skimage import exposure
import numpy as np


class advanced_filtering_addin_preprocessing(AddIn):
    """
        Blueprint for developing own methods and inserting own variables
        into the already existing PIV GUI via the AddIn system
    """

    # description for the Add_In_Handler textarea
    addin_tip = "This is the description of the advanced filter addin which " \
                "is still missing now"

    # has to be the add_in_name and its abbreviation
    add_in_name = "advanced_filtering_addin (afa)"

    # variables
    #########################################################
    # Place additional variables in the following sections. #
    # Widgets are created automatically. Don't care about   #
    # saving and restoring - new variables are included     #
    # automatically.                                        #
    #                                                       #
    # e.g.                                                  #
    #   **abbreviation**_**variable_name** =                #
    #       [**id over super group**, **variable_type**,    #
    #        **standard_value**,**hint**, **label**         #
    #        **tool tip**                                   #
    #########################################################
    variables = {
        'afa_frame': [2060, 'sub_labelframe', None, None, 'advanced filtering',
                      None],

        'afa_CLAHE': [2065, 'sub_bool', 'True', None, 'CLAHE filter',
                      'Contrast Limited Adaptive Histogram Equalization '
                      'filter (see skimage adapthist()).'],

        'afa_CLAHE_first': [2066, 'sub_bool', 'False', None,
                            'perform CLAHE before high pass',
                            'Perform CLAHE filter before Gaussian high pass '
                            'filters.'],

        'afa_CLAHE_auto_kernel': [2067, 'sub_bool', True, None,
                                  'automatic kernel sizing',
                                  'Have the kernel automatically sized to 1/8 '
                                  'width and height of the image.'],

        'afa_CLAHE_kernel': [2068, 'sub_int', 20, None, 'kernel size',
                             'Defining the size of the kernel for CLAHE.'],

        'afa_high_pass_filter_spacer': [2069, 'sub_h-spacer', None, None, None,
                                        None],

        'afa_high_pass_filter': [2074, 'sub_bool', 'False', None,
                                 'Gaussian high pass filter',
                                 'A simple subtracted Gaussian high pass '
                                 'filter.'],

        'afa_hp_sigma': [2075, 'sub_int', 5, None, 'sigma',
                         'Defining the sigma size of the subtracted gaussian '
                         'filter in the high pass filter '
                         '(positive ints only).'],

        'afa_hp_clip': [2076, 'sub_bool', 'True', None, 'clip at zero',
                        'Set all values less than zero to zero.'],

        'afa_intensity_threshold_spacer': [2080, 'sub_h-spacer', None, None,
                                           None, None],
        'afa_intensity_cap_filter': [2085, 'sub_bool', 'False', None,
                                     'intensity capping',
                                     'Simple global intesity cap filter. '
                                     'Masked pixels are set to the mean pixel '
                                     'intensity.'],

        'afa_ic_mult': [2086, 'sub_float', 2, None, 'std multiplication',
                        'Multiply the standard deviation of the pixel '
                        'intensities to get a higher cap value.'],

        'afa_Gaussian_lp_spacer': [2090, 'sub_h-spacer', None, None, None,
                                   None],

        'afa_gaussian_filter': [2095, 'sub_bool', 'False', None,
                                'Gaussian filter',
                                'Standard Gaussian blurring filter'
                                ' (see scipy gaussian_filter()).'],

        'afa_gf_sigma': [2100, 'sub_int', 1, None, 'sigma',
                         'Defining the sigma size for gaussian blur filter.'],

        'afa_intensity_clip_spacer': [2105, 'sub_h-spacer', None, None, None,
                                      None],

        'afa_intensity_clip': [2110, 'sub_bool', 'False', None,
                               'intensity clip',
                               'Any intensity less than the threshold is set '
                               'to zero.'],

        'afa_intensity_clip_min': [2115, 'sub_int', 20, None, 'min intensity',
                                   'Any intensity less than the threshold is '
                                   'set to zero with respect to the resized '
                                   'image inntensities.']
    }

    def advanced_filtering_method(self, img, GUI):
        resize = GUI.p['img_int_resize']
        if GUI.p['afa_CLAHE'] or GUI.p['afa_high_pass_filter']:
            if GUI.p['afa_CLAHE_first']:
                if GUI.p['afa_CLAHE']:
                    if GUI.p['afa_CLAHE_auto_kernel']:
                        kernel = None
                    else:
                        kernel = GUI.p['afa_CLAHE_kernel']

                    img = exposure.equalize_adapthist(img,
                                                      kernel_size=kernel,
                                                      clip_limit=0.01,
                                                      nbins=256)

                if GUI.p['afa_high_pass_filter']:
                    low_pass = gaussian_filter(img,
                                               sigma=GUI.p['afa_hp_sigma'])
                    img -= low_pass

                    if GUI.p['afa_hp_clip']:
                        img[img < 0] = 0

            else:
                if GUI.p['afa_high_pass_filter']:
                    low_pass = gaussian_filter(img,
                                               sigma=GUI.p['afa_hp_sigma'])
                    img -= low_pass

                    if GUI.p['afa_hp_clip']:
                        img[img < 0] = 0

                if GUI.p['afa_CLAHE']:
                    if GUI.p['afa_CLAHE_auto_kernel']:
                        kernel = None
                    else:
                        kernel = GUI.p['afa_CLAHE_kernel']

                    img = exposure.equalize_adapthist(img,
                                                      kernel_size=kernel,
                                                      clip_limit=0.01,
                                                      nbins=256)

        # simple intensity capping
        if GUI.p['afa_intensity_cap_filter']:
            upper_limit = np.mean(img) + GUI.p['afa_ic_mult'] * img.std()
            img[img > upper_limit] = upper_limit

        # simple intensity clipping
        if GUI.p['afa_intensity_clip']:
            img *= resize
            lower_limit = GUI.p['afa_intensity_clip_min']
            img[img < lower_limit] = 0
            img /= resize

        if GUI.p['afa_gaussian_filter']:
            img = gaussian_filter(img, sigma=GUI.p['afa_gf_sigma'])

        return img

    def __init__(self, gui):
        super().__init__()
        # has to be the method which is implemented above
        gui.preprocessing_methods.update(
            {"advanced_filtering_addin_preprocessing":
             self.advanced_filtering_method})



from openpivgui.AddIns.AddIn import AddIn
import numpy as np
import openpiv.validation as piv_vld
from openpivgui.open_piv_gui_tools import create_save_vec_fname, save


class sig2noise_addin_postprocessing(AddIn):
    """
        Blueprint for developing own methods and inserting own variables
        into the already existing PIV GUI via the AddIn system
    """

    # description for the Add_In_Handler textarea
    addin_tip = "This is the description of the advanced filter addin which " \
                "is still missing now"

    # has to be the add_in_name and its abbreviation
    add_in_name = "sig2noise_addin (s2n)"

    # variables
    #########################################################
    # Place additional variables in the following sections. #
    # Widgets are created automatically. Don't care about   #
    # saving and restoring - new variables are included     #
    # automatically.                                        #
    #                                                       #
    # e.g.                                                  #
    #   **abbreviation**_**variable_name** =                #
    #       [**id over super group**, **variable_type**,    #
    #        **standard_value**,**hint**, **label**         #
    #        **tool tip**                                   #
    #########################################################
    variables = {
        's2n_vld_sig2noise':
        [6010, 'bool', False, None,
         'signal to noise ratio validation',
         'Validate the data based on the signal to nose ratio ' +
         'of the cross correlation.'],

            's2n_sig2noise_threshold':
                [6030, 'float', 1.05, None,
                 's2n threshold',
                 'Threshold for filtering based on signal to noise ' +
                 'ratio. Recommended value: between 1.05 and 1.1.'],

            's2n_horizontal_spacer11': [6035, 'h-spacer', None, None, None,
                                        None]
    }

    def sig2noise(self, gui, delimiter):
        """Filter vectors based on the signal to noise threshold.

        See:
            openpiv.validation.sig2noise_val()
        """
        result_fnames = []
        for i, f in enumerate(gui.p['fnames']):
            data = np.loadtxt(f)
            u, v, mask = piv_vld.sig2noise_val(
                data[:, 2], data[:, 3], data[:, 5],
                threshold=gui.p['s2n_sig2noise_threshold'])

            save_fname = create_save_vec_fname(
                path=f,
                postfix='_sig2noise')

            save(data[:, 0],
                 data[:, 1],
                 u, v,
                 data[:, 4] + mask,
                 sig2noise=data[:, 5],
                 filename=save_fname,
                 delimiter=delimiter)
            result_fnames.append(save_fname)
        return result_fnames

    def __init__(self, gui):
        super().__init__()
        # has to be the method which is implemented above
        gui.postprocessing_methods.update(
            {"sig2noise_addin_postprocessing":
             ['validation', 's2n_vld_sig2noise', self.sig2noise]})



from openpivgui.AddIns.AddIn import AddIn


class pandas_plotting_addin_plotting(AddIn):
    """
        Blueprint for developing own methods and inserting own variables
        into the already existing PIV GUI via the AddIn system
    """

    # description for the Add_In_Handler textarea
    addin_tip = "This is the description of the advanced filter addin which " \
                "is still missing now"

    # has to be the add_in_name and its abbreviation
    add_in_name = "pandas_plotting_addin (ppa)"

    # variables
    #########################################################
    # Place additional variables in the following sections. #
    # Widgets are created automatically. Don't care about   #
    # saving and restoring - new variables are included     #
    # automatically.                                        #
    #                                                       #
    # e.g.                                                  #
    #   **abbreviation**_**variable_name** =                #
    #       [**id over super group**, **variable_type**,    #
    #        **standard_value**,**hint**, **label**         #
    #        **tool tip**                                   #
    #########################################################
    variables = {
        'ppa_pandas_frame':
        [8505, 'labelframe', None, None,
         'pandas', None],
            'ppa_plot_scaling':
                [8515, 'str', 'None', ('None', 'logx', 'logy', 'loglog'),
                 'axis scaling', 'scales the axes. logarithm scaling x-axis' +
                 ' --> logx; logarithm scaling y-axis --> logy; ' +
                 'logarithm scaling both axes --> loglog.'],
            'ppa_plot_xlim':
                [8525, 'str', '', None,
                 'limits for the x-axis',
                 'For implementation use (lower_limit, upper_limit).'],
            'ppa_plot_ylim':
                [8535, 'str', '', None,
                 'limits for the y-axis',
                 'For implementation use (lower_limit, upper_limit).'],
            'ppa_u_data':
                [8545, 'str', 'vx', None,
                 'x-data',
                 'Column name for the u-velocity component.' +
                 ' If unknown watch labbook entry.'],
            'ppa_v_data':
                [8555, 'str', 'vy', None,
                 'y-data',
                 'Column name for v-velocity component.' +
                 ' If unknown watch labbook entry.' +
                 ' For histogram only the v-velocity component is needed.'],
            'ppa_plot_grid':
                [8565, 'bool', True, None,
                 'grid',
                 'adds a grid to the diagram.'],
            'ppa_plot_legend':
                [8575, 'bool', True, None,
                 'legend',
                 'adds a legend to the diagram.'],

            'ppa_histogram_sub_frame':
                [8605, 'sub_labelframe', None, None, 'histogram', None],
            'ppa_histogram_quantity':
                [8615, 'sub', 'v_x', ('v', 'v_x', 'v_y'),
                 'histogram quantity',
                 'The absolute value of the velocity (v) or its x- ' +
                 'or y-component (v_x or v_y).'],
            'ppa_histogram_bins':
                [8625, 'sub_int', 20, None,
                 'histogram number of bins',
                 'Number of bins (bars) in the histogram.'],
            'ppa_histogram_normalize':
                [8635, 'sub_bool', False, None,
                 'normalize histogram',
                 'Normalize histogram (divide by the number of counts, '
                 'density).'],
            'ppa_histogram_type':
                [8645, 'sub', 'bar',
                 ('bar', 'barstacked', 'step', 'stepfilled'),
                 'histogram type',
                 'Choose histogram type. Only available for histogram' +
                 'plot.'],
    }

    def pandas_plot(self, data, parameter, figure):
        """
            Display a plot with the pandas plot utility.

            Parameters
            ----------
            data : pandas.DataFrame
                Data to plot.
            parameter : openpivgui.OpenPivParams.py
                Parameter-object.
            figure : matplotlib.figure.Figure
                An (empty) figure.

            Returns
            -------
            None.

        """
        # set boolean for chosen axis scaling
        if parameter['ppa_plot_scaling'] == 'None':
            logx, logy, loglog = False, False, False
        elif parameter['ppa_plot_scaling'] == 'logx':
            logx, logy, loglog = True, False, False
        elif parameter['ppa_plot_scaling'] == 'logy':
            logx, logy, loglog = False, True, False
        elif parameter['ppa_plot_scaling'] == 'loglog':
            logx, logy, loglog = False, False, True
        else:
            raise ValueError("non supported plot scaling chosen")
        # add subplot
        ax = figure.add_subplot(111)
        # set limits initially to None
        xlim = None
        ylim = None
        # try to set limits, if not possible (no entry) --> None
        try:
            xlim = (float(list(parameter['ppa_plot_xlim'].split(','))[0]),
                    float(list(parameter['ppa_plot_xlim'].split(','))[1]))
        except BaseException:
            pass
            # print('No Values or wrong syntax for x-axis limitation.')
        try:
            ylim = (float(list(parameter['ppa_plot_ylim'].split(','))[0]),
                    float(list(parameter['ppa_plot_ylim'].split(','))[1]))
        except BaseException:
            pass
            # print('No Values or wrong syntax for y-axis limitation.')
        # iteration to set value types to float
        for i in list(data.columns.values):
            data[i] = data[i].astype(float)

        if parameter['plot_type'] == 'histogram':
            # get column names as a list for comparing with chosen histogram
            # quantity
            col_names = list(data.columns.values)
            # if loop for histogram quantity
            if parameter['ppa_histogram_quantity'] == 'v_x':
                data_hist = data[col_names[2]]
            elif parameter['ppa_histogram_quantity'] == 'v_y':
                data_hist = data[col_names[3]]
            elif parameter['ppa_histogram_quantity'] == 'v':
                data_hist = (data[col_names[2]] ** 2 + data[
                    col_names[3]] ** 2) ** 0.5
            # histogram plot
            ax.hist(data_hist,
                    bins=int(parameter['ppa_histogram_bins']),
                    label=parameter['ppa_histogram_quantity'],
                    log=logy,
                    range=xlim,
                    density=parameter['ppa_histogram_normalize'],
                    histtype=parameter['ppa_histogram_type'],
                    )
            ax.grid(parameter['ppa_plot_grid'])
            ax.legend()
            ax.set_xlabel('velocity [m/s]')
            ax.set_ylabel('number of vectors')
            ax.set_title(parameter['plot_title'])
        else:
            data.plot(x=parameter['ppa_u_data'],
                      y=parameter['ppa_v_data'],
                      kind=parameter['plot_type'],
                      title=parameter['plot_title'],
                      grid=parameter['ppa_plot_grid'],
                      legend=parameter['ppa_plot_legend'],
                      logx=logx,
                      logy=logy,
                      loglog=loglog,
                      xlim=xlim,
                      ylim=ylim,
                      ax=ax)

    def __init__(self, gui):
        super().__init__()
        hint = gui.p.hint["plot_type"]
        hint = hint + ('histogram', 'line', 'density')
        gui.p.hint["plot_type"] = hint
        # has to be the method which is implemented above
        gui.plotting_methods.update(
            {"pandas_plotting_addin_plotting":
             ['plotting', 'pandas_plotting', ['histogram', 'line', 'density'],
              self.pandas_plot]})



from openpivgui.AddIns.AddIn import AddIn
from matplotlib.colors import LinearSegmentedColormap
import numpy as np
from copy import copy
from matplotlib import pyplot as plt


class streamlines_plotting_addin_plotting(AddIn):
    """
        Blueprint for developing own methods and inserting own variables
        into the already existing PIV GUI via the AddIn system
    """

    # description for the Add_In_Handler textarea
    addin_tip = "This is the description of the advanced filter addin which " \
                "is still missing now"

    # has to be the add_in_name and its abbreviation
    add_in_name = "streamlines_plotting_addin (spa)"

    # variables
    #########################################################
    # Place additional variables in the following sections. #
    # Widgets are created automatically. Don't care about   #
    # saving and restoring - new variables are included     #
    # automatically.                                        #
    #                                                       #
    # e.g.                                                  #
    #   **abbreviation**_**variable_name** =                #
    #       [**id over super group**, **variable_type**,    #
    #        **standard_value**,**hint**, **label**         #
    #        **tool tip**                                   #
    #########################################################
    variables = {
        'spa_streamline_frame': [8405, 'labelframe', None, None,
                                 'streamlines', None],
        'spa_density': [8415, 'str', '0.5, 1', None, 'streamline density',
                        'streamline density. Can be one value (e.g. 1) or '
                        'a couple of values for a range (e.g. 0.5, 1).'],
        'spa_color_map': [8425, 'str', 'viridis', ('viridis', 'jet',
                                                   'short rainbow',
                                                   'long rainbow',
                                                   'seismic',
                                                   'autumn',
                                                   'binary'),
                          'Color map', 'Color map for streamline- '
                          'and contour-plot.'],
        'spa_velocity_color': [8435, 'str', 'v', ('vx', 'vy', 'v'),
                               'set colorbar to: ',
                               'Set colorbar to velocity components.'],
        'spa_integrate_dir': [8445, 'str', 'both', ('both', 'forward',
                                                    'backward'),
                              'streamline direction',
                              'Integrate the streamline in forward, '
                              'backward or both directions. default is '
                              'both.'],
        'spa_vec_width': [8455, 'float', 0.0025, None, 'vector line width',
                          'Line width as a fraction of the plot width.'],
    }

    # creating a custom rainbow colormap
    short_rainbow = {'red': ((0.0, 0.0, 0.0),
                             (0.2, 0.2, 0.2),
                             (0.5, 0.0, 0.0),
                             (0.8, 1.0, 1.0),
                             (1.0, 1.0, 1.0)),
                     'green': ((0.0, 0.0, 0.0),
                               (0.2, 1.0, 1.0),
                               (0.5, 1.0, 1.0),
                               (0.8, 1.0, 1.0),
                               (1.0, 0.0, 0.0)),
                     'blue': ((0.0, 1.0, 1.0),
                              (0.2, 1.0, 1.0),
                              (0.5, 0.0, 0.0),
                              (0.8, 0.0, 0.0),
                              (1.0, 0.0, 0.0))}

    long_rainbow = {'red': ((0.0, 0.0, 0.0),
                            (0.1, 0.5, 0.5),
                            (0.2, 0.0, 0.0),
                            (0.3, 0.2, 0.2),
                            (0.5, 0.0, 0.0),
                            (0.7, 1.0, 1.0),
                            (0.8, 1.0, 1.0),
                            (1.0, 1.0, 1.0)),
                    'green': ((0.0, 0.0, 0.0),
                              (0.1, 0.0, 0.0),
                              (0.2, 0.0, 0.0),
                              (0.3, 1.0, 1.0),
                              (0.5, 1.0, 1.0),
                              (0.7, 1.0, 1.0),
                              (0.8, 0.0, 0.0),
                              (1.0, 0.3, 0.3)),
                    'blue': ((0.0, 0.0, 0.0),
                             (0.1, 0.5, 0.5),
                             (0.2, 1.0, 1.0),
                             (0.3, 1.0, 1.0),
                             (0.5, 0.0, 0.0),
                             (0.7, 0.0, 0.0),
                             (0.8, 0.0, 0.0),
                             (1.0, 1.0, 1.0))}

    short_rainbow = LinearSegmentedColormap('my_colormap', short_rainbow, 256)
    long_rainbow = LinearSegmentedColormap('my_colormap', long_rainbow, 256)

    def streamlines(self, data, parameter, figure):
        """
            Display a streamline plot.

            Parameters
            ----------
            data : pandas.DataFrame
                Data to plot.
            parameter : openpivgui.OpenPivParams.py
                Parameter object.
            figure : matplotlib.figure.Figure
                An (empty) Figure object.
        """
        ax = figure.add_subplot(111)

        # make sure all values are from type float
        for i in list(data.columns.values):
            data[i] = data[i].astype(float)

        # get density for streamline plot.
        try:
            density = (float(list(parameter['spa_density'].split(','))[0]),
                       float(list(parameter['spa_density'].split(','))[1]))
        except BaseException:
            density = float(parameter['spa_density'])

        # Choosing the correct colormap
        if parameter['spa_color_map'] == 'short rainbow':
            colormap = self.short_rainbow
        elif parameter['spa_color_map'] == 'long rainbow':
            colormap = self.long_rainbow
        else:
            colormap = parameter['spa_color_map']

        # pivot table for streamline plot
        data_vx = data.pivot(index='y',
                             columns='x',
                             values='vx')
        data_vy = data.pivot(index='y',
                             columns='x',
                             values='vy')

        # choosing data for the colormap
        if parameter['spa_velocity_color'] == 'vx':
            color_values = data_vx.values
        elif parameter['spa_velocity_color'] == 'vy':
            color_values = data_vy.values
        else:
            color_values = (data_vx.values ** 2 + data_vy.values ** 2) ** 0.5

        # try to create streamline plot. If values are not equally spaced the
        # exception will space the values equally (mean difference is
        # calculated.)

        try:
            fig = ax.streamplot(data_vx.columns,
                                data_vx.index,
                                data_vx.values,
                                data_vy.values,
                                density=density,
                                color=color_values,
                                cmap=colormap,
                                integration_direction=parameter[
                                    'spa_integrate_dir'],
                                linewidth=parameter['spa_vec_width'])
        except BaseException:
            # get dimension of the DataFrame
            dim = [len(set(data.x)), len(set(data.y))]

            # calculate mean difference for x and y values
            diff = [round(np.mean([data.x[i + 1] - data.x[i]
                                   for i in range(dim[0] - 1)]), 6),
                    round(
                        np.mean([data.y[dim[0] * (i + 1)] - data.y[dim[0] * i]
                                 for i in range(dim[1] - 1)]), 6)]

            # this list is initialized with starting values and will be
            # added by
            # equally spaced values.
            cache = [round(copy(data.x[0]), 6), round(copy(data.y[0]), 6)]

            # nested lists with equally spaced coordinates
            coordinates = [[], []]

            # loop for calculating the new x data
            j = 1
            for i in range(1, len(data)):
                if i == dim[0] * j:
                    coordinates[0].append(round(cache[0], 6))
                    cache[0] = coordinates[0][0]
                    j += 1
                else:
                    coordinates[0].append(round(cache[0], 6))
                    cache[0] += diff[0]
            coordinates[0].append(round(cache[0], 6))

            # loop for calculating the new y data
            j = 1
            for i in range(len(data)):
                if i == dim[0] * j:
                    cache[1] += diff[1]
                    coordinates[1].append(round(cache[1], 6))
                    j += 1
                else:
                    coordinates[1].append(round(cache[1], 6))

            # overwrite the old x and y values with the new ones
            data.x = coordinates[0]
            data.y = coordinates[1]

            # create new pivot tables for streamline plot
            data_vx = data.pivot(index='y', columns='x', values='vx')
            data_vy = data.pivot(index='y', columns='x', values='vy')

            # choosing data for the colormap
            if parameter['spa_velocity_color'] == 'vx':
                color_values = data_vx.values
            elif parameter['spa_velocity_color'] == 'vy':
                color_values = data_vy.values
            else:
                color_values = (data_vx.values ** 2
                                + data_vy.values ** 2) ** 0.5

            # new streamline plot with equally spaced coordinates
            fig = ax.streamplot(data_vx.columns,
                                data_vx.index,
                                data_vx.values,
                                data_vy.values,
                                density=density,
                                color=color_values,
                                cmap=colormap,
                                integration_direction=parameter[
                                    'spa_integrate_dir'],
                                linewidth=parameter['spa_vec_width'])
        # add colorbar
        cb = plt.colorbar(fig.lines, ax=ax)
        cb.ax.set_ylabel('Velocity [m/s]')

        # set origin to top left or bottom left
        if parameter['invert_yaxis']:
            ax.set_ylim(ax.get_ylim()[::-1])

        # add diagram options
        ax.set_xlabel('x-position')
        ax.set_ylabel('y-position')
        ax.set_title(parameter['plot_title'])

    def __init__(self, gui):
        super().__init__()
        hint = gui.p.hint["plot_type"]
        hint = hint + ('streamlines',)
        gui.p.hint["plot_type"] = hint
        # has to be the method which is implemented above
        gui.plotting_methods.update(
            {"streamlines_plotting_addin_plotting":
             ['plotting', 'streamlines_plotting', ['streamlines'],
              self.streamlines]})
